package org.rcisoft.business.device.assets.controller;


/*固定导入*/

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.poi.hssf.usermodel.*;
import org.rcisoft.business.device.assets.entity.BusDevice;
import org.rcisoft.business.device.assets.service.BusDeviceService;
import org.rcisoft.business.device.assets.vo.DeviceAssetStatisticVo;
import org.rcisoft.business.device.assets.vo.DeviceParamsVo;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.core.controller.PaginationController;
import org.rcisoft.core.model.GridModel;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.Result;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.propertyeditors.CustomDateEditor;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.WebDataBinder;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.context.request.WebRequest;

import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.IOException;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
* Created by  on 2018-4-12 15:33:23.
*/
@Api(tags = "项目评估--参数库")
@RestController
@RequestMapping("busdevice")
public class BusDeviceController extends PaginationController<BusDevice> {

    @Autowired
    private BusDeviceService busDeviceServiceImpl;


    @ApiOperation(value="添加设备", notes="添加设备")
    @RequestMapping(value = "/add")
    public Result add(@Valid BusDevice busDevice) {
        PersistModel data = busDeviceServiceImpl.save(busDevice);
        return Result.builder(data);
    }


    @ApiOperation(value="修改设备", notes="修改设备")
    @RequestMapping("/update")
    public Result update(@Valid BusDevice busDevice) {
        PersistModel data = busDeviceServiceImpl.merge(busDevice);
        return Result.builder(data);
    }

    @ApiOperation(value="删除设备", notes="删除设备")
    @RequestMapping("/delete")
    public Result delete(@RequestParam String devid) {
        PersistModel data = busDeviceServiceImpl.delete(devid);
        return Result.builder(data);
    }

    @ApiOperation(value="查看单个设备", notes="查看单个设备")
    @RequestMapping("/detail")
    public Result detail(@RequestParam String devid) {
        return Result.builder(new PersistModel(1),
            MessageConstant.MESSAGE_ALERT_SUCCESS,
            MessageConstant.MESSAGE_ALERT_ERROR,
        busDeviceServiceImpl.findById(devid));
    }

    @ApiOperation(value="根据设备编号查看单个设备", notes="根据设备编号查看单个设备")
    @RequestMapping("/detailByDevNum")
    public Result detailByDevNum(@RequestParam String devNum) {
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                busDeviceServiceImpl.findByDevNum(devNum));
    }

    @ApiOperation(value="查看设备列表", notes="查看设备列表")
    @RequestMapping(value = "/queryBusDeviceByPagination")
    public GridModel listByPagination(BusDevice busDevice) {
        busDeviceServiceImpl.findAllByPagination(getPaginationUtility(), busDevice);
        return getGridModelResponse();
    }

    /**
     * 通过项目id和系统id获取该子系统中的所有设备信息
      * @param sysId    系统id
     * @param proId     项目id
     * @return
     */
    @ApiOperation(value = "通过项目id与子系统id获取子系统下所有的设备")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "sysId", value = "sysId", required = true,paramType = "query", dataType = "varchar"),
            @ApiImplicitParam(name = "proId", value = "proId", required = true,paramType = "query", dataType = "varchar")
            })
    @ResponseBody
    @RequestMapping(value = "/getDeviceBySysId",method = RequestMethod.GET)
    public Result getDeviceBySysId(@RequestParam("sysId") String sysId,
                                   @RequestParam("proId") String proId)
    {
        return Result.builder(new PersistModel(1), MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                busDeviceServiceImpl.getDevicesBySysId(sysId, proId));
    }

    @ApiOperation(value = "通过项目id设备清单excel文件")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "proId", value = "proId", required = true,paramType = "path", dataType = "varvhar")
    })
    @ResponseBody
    @RequestMapping(value = "/exportExcel/{proId}",method = RequestMethod.GET)
    public void exportExcel(HttpServletResponse response,
                            @PathVariable("proId") String proId) throws UnsupportedEncodingException {
        List<DeviceAssetStatisticVo> deviceList = new ArrayList<>();
        //先获取数据库数据
        deviceList = busDeviceServiceImpl.getDevicesBySysId(null, proId);
        //通过项目id获取对应的名称组拼成文件名
        String filename = busDeviceServiceImpl.getProNameByProId(proId) +"设备统计报表.xls";

        //创建excel生成对象
        HSSFWorkbook workbook = new HSSFWorkbook();
        HSSFSheet sheet = workbook.createSheet("设备统计表");

        //表头内容
        String[] headers = { "设备名称", "设备类型", "设备型号", "设备厂家","设备参数","安装时间"};
        HSSFRow row = sheet.createRow(0);
        //在excel表中添加表头
        for(int i=0;i<headers.length;i++){
            HSSFCell cell = row.createCell(i);
            HSSFRichTextString text = new HSSFRichTextString(headers[i]);
            cell.setCellValue(text);
        }
        //日期类型格式化
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm");
        //在表中存放查询到的数据放入对应的列
        int rowNum = 1;
        for (DeviceAssetStatisticVo temp :
                deviceList) {
            HSSFRow rowTemp = sheet.createRow(rowNum);
            rowTemp.createCell(0).setCellValue(temp.getDevNm());
            rowTemp.createCell(1).setCellValue(temp.getDevTpNm());
            rowTemp.createCell(2).setCellValue(temp.getModel());
            rowTemp.createCell(3).setCellValue(temp.getFacNm());
            rowTemp.createCell(4).setCellValue(temp.getParam());
            rowTemp.createCell(5).setCellValue(sdf.format(temp.getInstallDate()));
            rowNum++;
        }
        //设置响应头信息
        response.setHeader("Content-disposition", "attachment;filename="+URLEncoder.encode(filename, "utf-8"));
        try {
            OutputStream outputStream = response.getOutputStream();
            workbook.write(outputStream);
            outputStream.flush();
            outputStream.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
    @InitBinder
    public void initBinder(WebDataBinder webDataBinder, WebRequest webRequest) {
        DateFormat dateFormat=new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        webDataBinder.registerCustomEditor(Date.class, new CustomDateEditor(dateFormat, true));
    }

    @ApiOperation(value="参数库导出", notes="参数库导出功能，需要将页面上的名称、编号以json对象数组的形式传到后台")
    @RequestMapping("/downloadExcel")
    public void downloadExcel(HttpServletResponse response, @RequestBody List<DeviceParamsVo> list) {
        busDeviceServiceImpl.downloadExcel(response, list);
    }

    @ApiOperation(value="生成设备二维码", notes="生成设备二维码")
    @RequestMapping("/generateCode")
    public Result generateCode(@RequestParam String devNum) {
        return Result.builder(busDeviceServiceImpl.generateCode(devNum));
    }

}
