package org.rcisoft.business.device.assets.dao;

import org.apache.ibatis.annotations.*;
import org.rcisoft.business.device.assets.vo.DeviceAssetStatisticVo;
import org.rcisoft.core.base.BaseMapper;
import org.rcisoft.business.device.assets.entity.BusDevice;
import org.springframework.stereotype.Repository;


import java.util.List;
import java.util.Map;

/**
* Created with  on 2018-4-12 15:33:23.
*/
@Repository
public interface BusDeviceRepository extends BaseMapper<BusDevice> {

    /**
    * 分页查询 busDevice
    *
    */
    @Select("<script>select a.*,b.FAC_NM,DATE_FORMAT(a.install_date,'%Y-%m-%d %H:%i:%s') INSTALL_DAY,c.DEV_TP_NM from bus_device a left join bus_factory b on a.fac_id = b.fac_id" +
            " left join bus_device_tp c on a.dev_tp_id = c.dev_tp_id " +
            " where 1=1" +
            "<if test='proId != null'> and a.pro_id = #{proId}</if>" +
            "<if test='devTpId != null'> and a.dev_tp_id = #{devTpId}</if>" +
            "<if test='devNum != null'> and a.dev_num = #{devNum}</if>" +
            "<if test='qrcodeUrl != null'> and a.qrcode_url is not null</if>"
            + "</script>")
    List<Map<String,Object>> queryBusDevices(BusDevice busDevice);

    /*
    通过项目id与子系统id获取子系统中的所有设备
    @param sysId  子系统id
    @param proId  项目id
     */
    List<DeviceAssetStatisticVo> getBusDevicesBySysId(@Param("sysId") String sysId, @Param("proId") String proId);

    /**
     * 通过项目id获取项目名称
     * @param proId
     * @return
     */
    String getProNameByProId(@Param("proId") String proId);

    /**
     * 通过系统id获取系统名称
      * @param sysId
     * @return
     */
    String getSysNameBySysId(@Param("sysId") String sysId);

    /**
     * 查询设备是否有原始数据
     * @param map
     * @return
     */
    @Select("<script>select a.tm from total_original a,bus_device b where a.dev_num = b.dev_num and  b.dev_id = #{dev_id} limit 1</script>")
    List<Map<String,Object>> queryOriginalByDev(Map<String,Object> map);

    /**
     * 根据设备编号查询设备信息
     * @param devNum
     * @return
     */
    @Select("<script>select * from bus_device where dev_num = #{devNum}</script>")
    List<Map<String,Object>> queryDeviceByDevNum(@Param("devNum") String devNum);

    /**
     * 查询所有项目中，所有设备的：设备id，最低耗电量，运行时长，耗电量
     * @param time
     * @return
     */
    @Select("<script>select d.DEV_ID,d.SHUT_POWER,d.RUN_TM,sum(e.elec) ELEC from bus_device d,bus_device_meter m,energy_count_m e " +
            "where d.DEV_NUM=m.DEV_NUM and m.MET_NUM=e.MET_NUM and date_format(e.CREATE_TIME, '%Y-%m-%d %H:%i')=#{time}" +
            "group by d.DEV_ID,d.SHUT_POWER,d.RUN_TM</script>")
    List<Map<String, Object>> addRunTime(@Param("time") String time);

    /**
     * 批量更新设备的运行时长
     * @param list
     * @return
     */
    @Update("<script><foreach collection=\"list\" item=\"item\" separator=\";\">" +
            "update bus_device set RUN_TM=#{item.runTm} where DEV_ID=#{item.devId}" +
            "</foreach></script>")
    Integer batchUpdateRunTm(List<BusDevice> list);

    /**
     * 查询该项目下所有设备的故障数量和运行时长
     * @param proId
     * @return
     */
    @Select("<script>select DEV_NM,RUN_TM,ERROR_NUM from bus_device where PRO_ID=#{proId}</script>")
    @ResultMap(value = "BaseResultMap")
    List<BusDevice> selectErrorandRuntime(@Param("proId") String proId);
}

