package org.rcisoft.business.device.assets.service.impl;

import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.CellType;
import com.google.zxing.WriterException;
import org.rcisoft.business.device.assets.dao.BusDeviceRepository;
import org.rcisoft.business.device.assets.service.BusDeviceService;
import org.rcisoft.business.device.assets.vo.DeviceAssetStatisticVo;
import org.rcisoft.business.device.assets.vo.DeviceParamsVo;
import org.rcisoft.business.manage.dao.BusLibraryParamRepository;
import org.rcisoft.business.manage.entity.BusDevicetpParam;
import org.rcisoft.business.manage.entity.BusLibraryParam;
import org.rcisoft.business.manage.service.BusDeviceParamService;
import org.rcisoft.business.manage.service.BusDevicetpParamService;
import org.rcisoft.business.manage.service.BusLibraryParamService;
import org.rcisoft.core.aop.PageUtil;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.business.device.assets.entity.BusDevice;

import org.rcisoft.core.util.QRCodeUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;


import java.io.IOException;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.math.BigDecimal;
import java.net.URLEncoder;
import java.nio.charset.Charset;
import java.text.SimpleDateFormat;
import java.util.*;

import lombok.extern.slf4j.Slf4j;
import tk.mybatis.mapper.entity.Example;

import javax.servlet.http.HttpServletResponse;

/**
 * Created by  on 2018-4-12 15:33:23.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class BusDeviceServiceImpl implements BusDeviceService {

    @Autowired
    private BusDeviceRepository busDeviceRepository;
    @Autowired
    private BusDeviceParamService busDeviceParamService;
    @Autowired
    private BusLibraryParamRepository busLibraryParamRepository;
    @Autowired
    private BusDevicetpParamService busDevicetpParamService;

    /**
     * 二维码存放路径
     */
    @Value("${filepath.qrcode}")
    String qrcodePath;

    /**
    * 保存 busDevice
    * @param busDevice
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel save(BusDevice busDevice){
        int line = 1;
        String message = "";
        //增加操作
        busDevice.setDevId(UUID.randomUUID().toString().replace("-",""));
        boolean flag = busDeviceRepository.queryDeviceByDevNum(busDevice.getDevNum())!=null && busDeviceRepository.queryDeviceByDevNum(busDevice.getDevNum()).size()>0;
        if(flag){
            line = 0;
            message = "设备编号已存在，新增失败";
        }else{
            // 保存二维码路径（配置的路径+项目编号）
            String path = qrcodePath + busDevice.getProId() + "/";
            try {
                // 生成二维码(第一个参数（二维码数据）：设备编号，第二个参数（保存的路径）：保存的路径，第三个参数（文件名）：设备id)
                QRCodeUtils.createQRCodeFile(busDevice.getDevNum(), path, busDevice.getDevId());
            } catch (WriterException e) {
                e.printStackTrace();
            } catch (IOException e) {
                e.printStackTrace();
            }
            // 将二维码路径设置到对象中（项目id/设备id.JPG）
            busDevice.setQrcodeUrl(busDevice.getProId() + "/" + busDevice.getDevId() + ".JPG");
            line = busDeviceRepository.insertSelective(busDevice);
        }

        return new PersistModel(line, message);
    }


    /**
    * 修改 busDevice
    * @param busDevice
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel merge(BusDevice busDevice){
        int line = 0;
        String message = "";
        Example example = new Example(BusDevice.class);
        Example.Criteria criteria = example.createCriteria();
        criteria.andEqualTo("devId",busDevice.getDevId());
        if(busDevice.getDevId()!=null && !busDevice.getDevId().equals("")){
            line = busDeviceRepository.updateByExampleSelective(busDevice,example);
        }else{
            line = 0;
            message = "设备ID为空，更新失败";
        }

        return new PersistModel(line,message);
    }

    /**
     * 删除
     * @param devid
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel delete(String devid) {
        int line = 0;
        String message = "";
        Example example = new Example(BusDevice.class);
        Example.Criteria criteria = example.createCriteria();
        criteria.andEqualTo("devId",devid);
        Map<String,Object> map = new HashMap<>();
        map.put("dev_id",devid);
        if(devid != null && !devid.equals("")){
            if(busDeviceRepository.queryOriginalByDev(map)==null || busDeviceRepository.queryOriginalByDev(map).size()<1){
                line = busDeviceRepository.deleteByExample(example);
            }else{
                message = "设备存在关联数据，删除失败";
            }
        }else{
            message = "设备ID为空，删除失败";
        }

        return new PersistModel(line,message);
    }

    /**
    * 根据id查询 busDevice
    * @param id
    * @return
    */
    @Override
    public List<BusDevice> findById(String id){
        Example example = new Example(BusDevice.class);
        Example.Criteria criteria = example.createCriteria();
        criteria.andEqualTo("devId",id);
        return busDeviceRepository.selectByExample(example);
    }

    @Override
    public List<Map<String,Object>> findByDevNum(String devNum){
        BusDevice busDevice = new BusDevice();
        busDevice.setDevNum(devNum);
        return busDeviceRepository.queryBusDevices(busDevice);
    }

    /**
    * 分页查询 busDevice
    * @param busDevice
    * @return
    */
    @Override
    public List<Map<String,Object>> findAllByPagination(PageUtil<BusDevice> paginationUtility,
                                               BusDevice busDevice){
        //避免查出所有设备
        if(busDevice.getProId()!=null && !busDevice.getProId().equals("")){
            List<Map<String,Object>> list = busDeviceRepository.queryBusDevices(busDevice);
            for(Map<String,Object> tmap : list){
                tmap.put("paramList",busDeviceParamService.findByDev(tmap.get("DEV_NUM").toString(),tmap.get("PRO_ID").toString()));
                tmap.put("libraryparam",busLibraryParamRepository.queryLibraryParamByDev(tmap.get("DEV_NUM").toString()));
                BusDevicetpParam busDevicetpParam = new BusDevicetpParam();
                busDevicetpParam.setProId(tmap.get("PRO_ID").toString());
                busDevicetpParam.setDevTpId(tmap.get("DEV_TP_ID").toString());
                tmap.put("deviceTpParam",busDevicetpParamService.queryBusDevicetpParam(busDevicetpParam));
            }
            return list;
        }
        return null;
    }
    /*
        通过项目id与子系统id获取子系统中的所有设备
        @param sysId  子系统id
        @param proId  项目id
         */
    @Override
    public List<DeviceAssetStatisticVo> getDevicesBySysId(String sysId, String proId) {
        return busDeviceRepository.getBusDevicesBySysId(sysId,proId);
    }

    @Override
    public String getProNameByProId(String proId) {
        return busDeviceRepository.getProNameByProId(proId);
    }

    @Override
    public String getSysNameBySysId(String sysId) {
        return busDeviceRepository.getSysNameBySysId(sysId);
    }

    @Transactional
    @Override
    public Integer addRunTime() {
        //最后需要保存的数据
        List<BusDevice> saveList = new ArrayList<>();
        //得到当前时间前10分钟的时间
        String time = new SimpleDateFormat("yyyy-MM-dd HH:mm").format(new Date(System.currentTimeMillis() - 600000));
        //查询10分钟前所有数据的耗电量信息
        List<Map<String, Object>> list = busDeviceRepository.addRunTime(time);
        list.forEach(map -> {
            BigDecimal shutPower = (BigDecimal) map.get("SHUT_POWER");
            shutPower = shutPower == null ? new BigDecimal(0) : shutPower;
            BigDecimal elec = (BigDecimal) map.get("ELEC");
            elec = elec == null ? new BigDecimal(0) : elec;
            //实际电量 > 关闭电量
            if (elec.compareTo(shutPower) == 1) {
                BusDevice busDevice = new BusDevice();
                busDevice.setDevId((String) map.get("DEV_ID"));
                BigDecimal runTm = (BigDecimal) map.get("RUN_TM");
                runTm = runTm == null ? new BigDecimal(0) : runTm;
                runTm = runTm.add(new BigDecimal(600));
                busDevice.setRunTm(runTm);
                saveList.add(busDevice);
            }
        });
        //更新结果
        if (saveList.size() > 0)
            return busDeviceRepository.batchUpdateRunTm(saveList);
        else
            return 0;
    }

    @Override
    public void downloadExcel(HttpServletResponse response, List<DeviceParamsVo> list) {
        try (OutputStream outputStream = response.getOutputStream()) {
            response.setHeader("Content-disposition", "attachment;filename=download.xls");
            outputStream.write(this.createExcel(list));
            outputStream.flush();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    @Override
    public PersistModel generateCode(String devNum) {
        int line = 1;
        String message = "";
        boolean flag = busDeviceRepository.queryDeviceByDevNum(devNum)!=null && busDeviceRepository.queryDeviceByDevNum(devNum).size()>0;
        if(!flag){
            line = 0;
            message = "设备编号不存在，生成失败";
        }else{
            // 保存二维码路径（配置的路径+项目编号）
            String path = qrcodePath + "/";
            try {
                // 生成二维码(第一个参数（二维码数据）：设备编号，第二个参数（保存的路径）：保存的路径，第三个参数（文件名）：设备id)
                QRCodeUtils.createQRCodeFile(devNum, path,devNum);
            } catch (WriterException e) {
                e.printStackTrace();
            } catch (IOException e) {
                e.printStackTrace();
            }
            Example example = new Example(BusDevice.class);
            Example.Criteria criteria = example.createCriteria();
            criteria.andEqualTo("devNum",devNum);
            BusDevice busDevice = new BusDevice();
            busDevice.setDevNum(devNum);
            busDevice.setQrcodeUrl(devNum + ".JPG");
            line = busDeviceRepository.updateByExampleSelective(busDevice,example);
        }
        return new PersistModel(line,message);
    }

    private byte[] createExcel(List<DeviceParamsVo> list) throws IOException {
        HSSFWorkbook workbook = null;
        try(ByteArrayOutputStream os = new ByteArrayOutputStream()) {
            workbook = new HSSFWorkbook();
            HSSFSheet sheet = workbook.createSheet("sheet1");
            //表头
            String[] title = new String[]{"名称", "编号"};
            HSSFRow title_row = sheet.createRow(0);
            for (int i = 0; i < title.length; i++) {
                title_row.createCell(i, CellType.STRING).setCellValue(title[i]);
            }
            //内容
            for (int i = 0; i < list.size(); i++) {
                HSSFRow row = sheet.createRow(i + 1);
                DeviceParamsVo deviceParamsVo = list.get(i);
                row.createCell(0, CellType.STRING).setCellValue(deviceParamsVo.getName());
                row.createCell(1, CellType.STRING).setCellValue(deviceParamsVo.getNum());
            }
            workbook.write(os);
            return os.toByteArray();
        } finally {
            workbook.close();
        }
    }

}
