package org.rcisoft.business.device.inspection.controller;


/*固定导入*/

import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.poi.hssf.usermodel.*;
import org.rcisoft.business.device.assets.vo.DeviceAssetStatisticVo;
import org.rcisoft.business.device.inspection.service.InspectionService;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.Result;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
* Created by  on 2018-4-18 15:41:54.
*/
@RestController
@RequestMapping("InspectionEvaluate")
public class InspectionController {
    @Autowired
    private InspectionService inspectionService;

    @ApiOperation(value = "通过项目查询最新的项目巡检记录")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "proId", value = "项目id", required = true,paramType = "path", dataType = "varvhar")
    })
    @ResponseBody
    @GetMapping(value = "/listTheLastInspection/{proId}")
    public Result listTheLastInspection(@PathVariable("proId") String proId){
        return Result.builder(new PersistModel(1), MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                inspectionService.listTheLastInspectionInfomatinByProIdAndFlag(proId));
    }


    @ApiOperation(value = "查询该项目下某个设备的所有巡检记录")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "proId", value = "项目id", required = true,paramType = "path", dataType = "varvhar"),
            @ApiImplicitParam(name = "devNum", value = "设备编号", required = true,paramType = "path", dataType = "varvhar"),
            @ApiImplicitParam(name = "mon", value = "年月", required = true,paramType = "path", dataType = "varvhar")
    })
    @ResponseBody
    @GetMapping(value = "/listAllInspection/{proId}/{devNum}/{mon}")
    public Result listAllInspection(@PathVariable("proId") String proId, @PathVariable("devNum") String devNum, @PathVariable("mon") String mon){
        return Result.builder(new PersistModel(1), MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                inspectionService.listAllInspectionInfomatinByProIdAndDevNum(proId, devNum,mon));
    }

    @ApiOperation(value = "导出单个设备的月巡检记录")
    @ResponseBody
    @RequestMapping(value = "/exportExcel",method = RequestMethod.GET)
    public void exportExcel(HttpServletResponse response,
                            @RequestParam String proId, @RequestParam("devNum") String devNum, @RequestParam("mon") String mon) throws UnsupportedEncodingException {
        List<Map<String, Object>> deviceList = new ArrayList<>();
        //先获取数据库数据
        deviceList = inspectionService.listAllInspectionInfomatinByProIdAndDevNum(proId, devNum,mon);
        //通过项目id获取对应的名称组拼成文件名
        String filename = devNum + "-"+ mon + "巡检记录.xls";

        //创建excel生成对象
        HSSFWorkbook workbook = new HSSFWorkbook();
        HSSFSheet sheet = workbook.createSheet(mon+"巡检记录表");

        //表头内容
        String[] headers = { "时间", "巡检人", "巡检结果"};
        HSSFRow row = sheet.createRow(0);
        //在excel表中添加表头
        for(int i=0;i<headers.length;i++){
            HSSFCell cell = row.createCell(i);
            HSSFRichTextString text = new HSSFRichTextString(headers[i]);
            cell.setCellValue(text);
        }
        //日期类型格式化
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm");
        //在表中存放查询到的数据放入对应的列
        int rowNum = 1;
        for (Map<String,Object> temp :
                deviceList) {
            HSSFRow rowTemp = sheet.createRow(rowNum);
            rowTemp.createCell(0).setCellValue(temp.get("INSP_TM").toString());
            rowTemp.createCell(1).setCellValue(temp.get("USER_NICK_NAME").toString());
            rowTemp.createCell(2).setCellValue(temp.get("INSP_CONTENT").toString());
            rowNum++;
        }
        //设置响应头信息
        response.setHeader("Content-disposition", "attachment;filename="+ URLEncoder.encode(filename, "utf-8"));
        try {
            OutputStream outputStream = response.getOutputStream();
            workbook.write(outputStream);
            outputStream.flush();
            outputStream.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }



    @ApiOperation(value="下载二维码", notes="下载二维码")
    @RequestMapping("/download")
    public void download(HttpServletResponse response, @RequestParam String filename) {
        inspectionService.download(response,filename);
    }

}
