package org.rcisoft.business.device.maintenance.controller;

import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.rcisoft.business.device.maintenance.entity.BusDevService;
import org.rcisoft.business.device.maintenance.service.MaintenanceService;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.Result;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;

/**
 * @author: WangXinYu
 * @describe: 设备————养护计划Controller
 * @create: 2018年04月20日 15:10
 **/
@RestController
@RequestMapping("maintenance")
public class MaintenanceController {
    @Autowired
    MaintenanceService maintenanceService;

    @ApiOperation(value = "添加维护记录", notes = "添加维护记录")
    @PostMapping(value = "/add")
    public Result add(@Valid BusDevService busDevService, BindingResult bindingResult) {
        int i = maintenanceService.saveBusDevService(busDevService);
        if (i > 0) {
            return Result.builder(new PersistModel(1),
                    MessageConstant.MESSAGE_ALERT_SUCCESS,
                    MessageConstant.MESSAGE_ALERT_ERROR,
                    busDevService);
        }
        return null;
    }
    @ApiOperation(value = "更新维护记录", notes = "更新维护记录")
    @PostMapping(value = "/update")
    public Result update(@Valid BusDevService busDevService, BindingResult bindingResult) {
        int i = maintenanceService.updateBusDevService(busDevService);
        if (i > 0) {
            return Result.builder(new PersistModel(1),
                    MessageConstant.MESSAGE_ALERT_SUCCESS,
                    MessageConstant.MESSAGE_ALERT_ERROR,
                    busDevService);
        }
        return null;
    }

    @ApiOperation(value = "删除维护记录", notes = "删除维护记录")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "serId", value = "serId", required = true, paramType = "query", dataType = "varchar"),
    })
    @DeleteMapping(value = "/delete")
    public Result delete(@RequestParam String serId) {
        int i = maintenanceService.deleteBusDevServiceById(serId);
        if (i > 0) {
            return Result.builder(new PersistModel(1),
                    MessageConstant.MESSAGE_ALERT_SUCCESS,
                    MessageConstant.MESSAGE_ALERT_ERROR,
                    serId);
        }
        return null;
    }

    @ApiOperation(value = "查询所有的设备种类", notes = "查询所有的设备种类")
    @GetMapping(value = "/listDeviceCategory")
    public Result listDeviceCategory() {
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                maintenanceService.listDeviceCategory());
    }

    @ApiOperation(value = "根据设备种类id查询该种类的设备", notes = "根据设备种类id查询该种类的设备")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "tpId", value = "tpId", required = true, paramType = "query", dataType = "varchar"),
    })
    @GetMapping(value = "/listDeviceByTpId")
    public Result listDeviceByTpId(@RequestParam String tpId) {
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                maintenanceService.listDeviceByTpId(tpId));

    }

    @ApiOperation(value = "根据日期（YYYY-mm-dd格式） 查询维修记录", notes = "根据日期（YYYY-mm-dd格式） 查询维修记录")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "date", value = "date", required = true, paramType = "query", dataType = "varchar"),
    })
    @GetMapping(value = "/listBusDevServiceByDate")
    public Result listBusDevServiceByDate(@RequestParam String date) {
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                maintenanceService.listBusDevServiceByDate(date));

    }

    @ApiOperation(value = "根据年份 查询维修记录", notes = "根据年份 查询维修记录")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "year", value = "year", required = true, paramType = "query", dataType = "varchar"),
    })
    @GetMapping(value = "/listBusDevServiceByYear")
    public Result listBusDevServiceByYear(@RequestParam String year) {
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                maintenanceService.listBusDevServiceByYear(year));

    }
    @ApiOperation(value = "根据年份 查询存在维修记录的日期", notes = "根据年份 查询存在维修记录的日期")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "year", value = "year", required = true, paramType = "query", dataType = "varchar"),
    })
    @GetMapping(value = "/listDevDate")
    public Result listDevDate(@RequestParam String year) {
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                maintenanceService.listDevDate(year));
    }
}
