package org.rcisoft.business.device.maintenance.dao;

import org.apache.ibatis.annotations.Delete;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;
import org.rcisoft.business.device.maintenance.entity.BusDevService;
import org.springframework.stereotype.Repository;
import tk.mybatis.mapper.common.BaseMapper;

import java.util.List;
import java.util.Map;

/**
 * The interface Maintenance repository.
 *
 * @author: WangXinYu
 * @describe: TODO
 * @create: 2018年04月20日 15:14
 */
@Repository
public interface MaintenanceRepository extends BaseMapper<BusDevService> {

    /**
     * List device category list.
     * 查询设备种类名
     *
     * @return the list
     */
    @Select("SELECT * FROM `bus_device_tp` ")
    List<Map<String,Object>> listDeviceCategory();

    /**
     * List device by tp id list.
     * 根据类型id 查询设备
     *
     * @param tpID the tp id
     * @return the list
     */
    @Select("SELECT * FROM bus_device WHERE DEV_TP_ID = #{tpID}")
    List<Map<String,Object>> listDeviceByTpId(@Param("tpID") String tpID);


    /**
     * List device inspect by date list.
     * 根据日期（YYYY-mm-dd格式） 查询维修记录
     *
     * @param date the date
     * @return the list
     */
    @Select("SELECT ds.*,d.DEV_NM,tp.DEV_TP_NM FROM bus_dev_service ds INNER JOIN bus_device d on d.DEV_ID = ds.DEV_ID INNER JOIN bus_device_tp tp ON tp.DEV_TP_ID=d.DEV_TP_ID WHERE DATE_FORMAT(SER_TM,'%Y-%m-%d') = #{date}")
    List<Map<String,Object>> listBusDevServiceByDate(@Param("date") String date);

    /**
     * List bus dev service by year list.
     * 根据年份 查询维修记录
     *
     * @param year the year
     * @return the list
     */
    @Select("SELECT ds.*,d.DEV_NM,tp.DEV_TP_NM,tp.DEV_TP_ID FROM bus_dev_service ds INNER JOIN bus_device d on d.DEV_ID = ds.DEV_ID INNER JOIN bus_device_tp tp ON tp.DEV_TP_ID=d.DEV_TP_ID WHERE DATE_FORMAT(SER_TM,'%Y') = #{year}")
    List<Map<String,Object>> listBusDevServiceByYear(@Param("year") String year);

    /**
     * Delete device inspect int.
     * 删除设备维修记录
     *
     * @param id the id
     * @return the int
     */
    @Delete("DELETE FROM bus_dev_service WHERE SER_ID = #{id}")
    int deleteBusDevServiceById(@Param("id") String id);


    /**
     * List dev date list.
     * 使用年份查询维护记录有的 日期
     * @param year the year
     * @return the list
     */
    @Select("SELECT DISTINCT  DATE_FORMAT(SER_TM,'%m') AS month,DATE_FORMAT(SER_TM,'%d') AS day FROM `bus_dev_service`")
    List<Map<String, Object>> listDevDate(@Param("year") String year);
}
