package org.rcisoft.business.device.maintenance.service.impl;

import lombok.extern.slf4j.Slf4j;
import org.rcisoft.business.device.maintenance.dao.MaintenanceRepository;
import org.rcisoft.business.device.maintenance.entity.BusDevService;
import org.rcisoft.business.device.maintenance.service.MaintenanceService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Map;
import java.util.UUID;

/**
 * @author: WangXinYu
 * @describe: TODO
 * @create: 2018年04月20日 15:32
 **/
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class    MaintenanceServiceImpl implements MaintenanceService {
    @Autowired
    MaintenanceRepository maintenanceRepository;

    /**
     *  插入维修记录
     * @param busDevService
     * @return
     */
    @Override
    public int saveBusDevService(BusDevService busDevService) {
        busDevService.setSerId(UUID.randomUUID().toString().replaceAll("-","").toUpperCase());
        int i= 0;
        try {
            i = maintenanceRepository.insertSelective(busDevService);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return i;
    }
    /**
     *  根据id修改维修记录
     * @param busDevService
     * @return
     */
    @Override
    public int updateBusDevService(BusDevService busDevService) {
        int i= 0;
        try {
            i = maintenanceRepository.updateByPrimaryKeySelective(busDevService);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return i;
    }

    /**
     * 删除设备维修记录
     * @param id
     * @return
     */
    @Override
    public int deleteBusDevServiceById(String id) {
        int i = maintenanceRepository.deleteBusDevServiceById(id);
        return i;
    }

    /**
     * 查询所有设备种类
     * @return busDeviceTps
     */
    @Override
    public List<Map<String,Object>> listDeviceCategory() {
        List<Map<String,Object>> busDeviceTps=maintenanceRepository.listDeviceCategory();
        return busDeviceTps;
    }

    /**
     * 根据设备种类id查询该种类所有的设备
     * @param tpID the tp id
     * @return busDevices
     */
    @Override
    public List<Map<String,Object>> listDeviceByTpId(String tpID) {
        List<Map<String,Object>> busDevices = maintenanceRepository.listDeviceByTpId(tpID);
        return busDevices;
    }

    /**
     * 根据日期（YYYY-mm-dd格式） 查询维修记录
     * @param date the date
     * @return
     */
    @Override
    public List<Map<String,Object>> listBusDevServiceByDate(String date) {
        List<Map<String,Object>> busDevServices = maintenanceRepository.listBusDevServiceByDate(date);
        return busDevServices;
    }

    /**
     * 根据 年份查询维修记录
     * @param year the year
     * @return
     */
    @Override
    public List<Map<String,Object>> listBusDevServiceByYear(String year) {
        List<Map<String,Object>> busDevServices = maintenanceRepository.listBusDevServiceByYear(year);
        return busDevServices;
    }

    /**
     * List dev date list.
     * 使用年份查询存在维护记录的 日期
     * @param year the year
     * @return the list
     */
    @Override
    public List<Map<String, Object>> listDevDate(String year) {
        List<Map<String,Object>> mapList=maintenanceRepository.listDevDate(year);
        return mapList;
    }
}
