package org.rcisoft.business.device.report.service.impl;

import com.alibaba.fastjson.JSONObject;
import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.CellType;
import org.rcisoft.business.device.report.dao.ReportRepository;
import org.rcisoft.business.device.report.entity.Original;
import org.rcisoft.business.device.report.entity.Report;
import org.rcisoft.business.device.report.service.ReportService;
import org.rcisoft.business.device.report.vo.DeviceVo;
import org.rcisoft.business.device.report.vo.Params;
import org.rcisoft.business.device.report.vo.ReportVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.IOException;
import java.io.OutputStream;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.List;

/**
 * Created by JiChao on 2018/5/3.
 */
@Service
public class ReportServiceImpl implements ReportService {

    @Value("${filepath.equipment}")
    private String filePath;

    @Autowired
    private ReportRepository reportRepository;

    @Override
    public void download(HttpServletResponse response, String proId, Integer year, Integer month, Integer day) {
        byte[] bytes = null;
        //如果有文件
        String fileName = filePath + proId + File.separator + year + File.separator + month + File.separator + day + ".xls";
        File file = new File(fileName);
        if (!file.getParentFile().exists()) file.getParentFile().mkdirs();
        try(OutputStream outputStream = response.getOutputStream()) {
            file = this.createExcel(proId, year, month, day, file);
            //转成字节
            bytes = FileUtils.readFileToByteArray(file);
            //设置下载头
            response.setHeader("Content-disposition", "attachment;filename=" + URLEncoder.encode(day + ".xls", "iso-8859-1"));
            outputStream.write(bytes);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * 查找，组合数据，生成文件
     * @param proId
     * @param year
     * @param month
     * @param day
     * @param file
     * @return
     */
    private File createExcel(String proId, Integer year, Integer month, Integer day, File file) {
        //查询条件
        Params p = new Params(proId, year, month, day, year + "-" + month + "-" + day);
        //得到设备的结果集
        List<ReportVo> reportVoList = this.getReportVoList(p);
        //得到数据的结果集
        List<Original> originalList = reportRepository.originalList(p);
        //---------------定义excel-----------------------
        HSSFWorkbook workbook = new HSSFWorkbook();
        //---------------定义excel-----------------------
        //循环设备结果集，每个设备创建一个sheet页
        reportVoList.forEach(reportVo -> {
            //定义sheet
            HSSFSheet sheet = workbook.createSheet(reportVo.getDevNm());
            //创建表头
            HSSFRow head = sheet.createRow(0);
            List<DeviceVo> deviceList = reportVo.getDeviceList();
            //第一列填时间
            head.createCell(0, CellType.STRING).setCellValue("时间");
            for (int i = 0; i < deviceList.size(); i++) {
                HSSFCell cell = head.createCell(i + 1, CellType.STRING);
                //表头填入参数名称
                cell.setCellValue(deviceList.get(i).getParamNm());
            }
            //定义一个增量
            int x = 1;
            //循环数据
            for (int i = 0; i < originalList.size(); i++) {
                Original original = originalList.get(i);
                String devNum = original.getDevNum();
                //判断devNum是否等于当前设备的devNum
                if (StringUtils.equals(devNum, reportVo.getDevNum())) {
                    //第一列放入时间
                    HSSFRow row = sheet.createRow(x++);
                    row.createCell(0, CellType.STRING).setCellValue(original.getTm());
                    //设备的参数对象
                    JSONObject originalJson = JSONObject.parseObject(original.getOriginalStr());
                    //传感器参数对象
                    JSONObject sensorJson = JSONObject.parseObject(original.getSensorJson());
                    //循环设备
                    for (int j = 0; j < deviceList.size(); j++) {
                        DeviceVo deviceVo = deviceList.get(j);
                        String param = deviceVo.getParam();//参数code
                        String psource = deviceVo.getPsource();//参数来源，1：设备 2：传感器
                        //需要保存的结果
                        String res = "";
                        if (psource.equals("1")) res = originalJson.get(param) == null ? "" : originalJson.get(param).toString();
                        else if (psource.equals("2")) res = sensorJson.get(param) == null ? "" : sensorJson.get(param).toString();
                        //依次放入行中
                        row.createCell(j + 1, CellType.STRING).setCellValue(res);
                    }
                    //用完删除当前行的数据
                    originalList.remove(i--);
                }
            }
        });
        try {
            workbook.write(file);
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                workbook.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return file;
    }

    /**
     * 转换结果集，变成设备的list，每个设备有自己的参数list
     * list套list形式
     * @param params proId
     * @return
     */
    private List<ReportVo> getReportVoList(Params params) {
        //所有设备参数的list
        List<Report> reportList = reportRepository.reportList(params);
        String _devNum = "";
        //返回的结果
        List<ReportVo> reportVoList = new ArrayList<>();
        //记录最后结果集的标签
        int num = -1;
        for (int i = 0; i < reportList.size(); i++) {
            Report report = reportList.get(i);
            String devNum = report.getDevNum();
            //不等于
            if (!StringUtils.equals(_devNum, devNum)){
                _devNum = devNum;
                ReportVo reportVo = new ReportVo();
                reportVo.setDevNm(devNum);
                reportVo.setDevNum(report.getDevNum());
                //加入新的集合中
                reportVoList.add(reportVo);
                num++;
            }
            //从集合里取到需要添加的记录，将当前的值放入
            ReportVo reportVo = reportVoList.get(num);
            List<DeviceVo> deviceVoList = reportVo.getDeviceList();
            DeviceVo deviceVo = new DeviceVo(report.getParam(), report.getPsource(), report.getParamNm());
            deviceVoList.add(deviceVo);
        }
        return reportVoList;
    }
}
